"""Code for validating files from different sources."""

import logging
from typing import FrozenSet

from cg_hermes.constants.workflow import CancerAnalysisType, Workflow
from cg_hermes.deliverables import Deliverables
from cg_hermes.models.tags import TagMap

LOG = logging.getLogger(__name__)


def get_deliverables_obj(
    deliverables: dict[str, list[dict[str, str]]],
    workflow: Workflow,
    analysis_type: CancerAnalysisType | None = None,
) -> Deliverables:
    if Workflow.BALSAMIC in workflow and not analysis_type:
        LOG.error(f"Please specify analysis type for {workflow}")
        raise SyntaxError
    return Deliverables(deliverables=deliverables, workflow=workflow, analysis_type=analysis_type)


def validate_tag_map(tag_map: dict[FrozenSet[str], dict]) -> bool:
    """Validate if a tag map is on the correct format."""
    for workflow_tags, value in tag_map.items():
        assert isinstance(workflow_tags, frozenset)
        TagMap.validate(value)
    return True



"""Tags that are defined in deliverables file mapped to tags used in CG

The tag sets that exists in all files are set to mandatory. Tag sets that exists in < 4 deliverables are not
mandatory by default. However, the tags that are available to a particular analysis is mandatory for that analysis.
"""

from cg_hermes.config.nextflow import NEXTFLOW_TAGS
from cg_hermes.constants.tags import (
    AlignmentTags,
    AnalysisTags,
    BioinfoToolsTags,
    FamilyTags,
    MipTags,
    NextflowTags,
    QCTags,
    RarediseaseTags,
    ReportTags,
    VariantTags,
    UsageTags,
)

RAREDISEASE_COMMON_TAGS = {
    frozenset(["alignment", "alignment_markduplicates"]): {
        "tags": [AlignmentTags.CRAM, ReportTags.PICARD_DUPLICATES],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["alignment", "alignment_markduplicates_index"]): {
        "tags": [AlignmentTags.CRAM_INDEX, ReportTags.PICARD_DUPLICATES],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["alignment", "alignment_mt"]): {
        "tags": [AlignmentTags.BAM_MT],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["alignment", "alignment_mt_index"]): {
        "tags": [AlignmentTags.BAM_MT_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["tiddit_coverage"]): {
        "tags": [AnalysisTags.TIDDIT_COVERAGE, AnalysisTags.BIGWIG],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["qc_bam", "mosdepth_d4"]): {
        "tags": [AnalysisTags.COVERAGE, ReportTags.D4],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["chromograph_cov", "tcov"]): {
        "tags": [BioinfoToolsTags.CHROMOGRAPH, AnalysisTags.TCOV],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["smncopynumbercaller", "tsv"]): {
        "tags": [VariantTags.SMN_CALLING],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["expansionhunter", "variant_catalog"]): {
        "tags": [BioinfoToolsTags.EXPANSIONHUNTER, MipTags.VARIANT_CATALOG],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["expansionhunter_stranger", "case_sv_str"]): {
        "tags": [VariantTags.VCF_STR],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["expansionhunter_stranger", "case_sv_str_index"]): {
        "tags": [VariantTags.VCF_STR_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["expansionhunter", "str_variants"]): {
        "tags": [BioinfoToolsTags.EXPANSIONHUNTER, VariantTags.VCF_STR],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["expansionhunter", "str_alignment"]): {
        "tags": [BioinfoToolsTags.EXPANSIONHUNTER, AlignmentTags.BAM],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["expansionhunter", "str_alignment_index"]): {
        "tags": [BioinfoToolsTags.EXPANSIONHUNTER, AlignmentTags.BAM_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["vcf2cytosure", "vcf2cytosure"]): {
        "tags": [VariantTags.VCF2CYTOSURE],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["gens_generatedata", "baf"]): {
        "tags": [BioinfoToolsTags.GENS, AnalysisTags.FRACSNP, AnalysisTags.BED],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["gens_generatedata", "baf_index"]): {
        "tags": [BioinfoToolsTags.GENS, AnalysisTags.FRACSNP, AnalysisTags.BED_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["gens_generatedata", "cov"]): {
        "tags": [BioinfoToolsTags.GENS, AnalysisTags.COVERAGE, AnalysisTags.BED],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["gens_generatedata", "cov_index"]): {
        "tags": [BioinfoToolsTags.GENS, AnalysisTags.COVERAGE, AnalysisTags.BED_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["call_mobile_elements", "call_mobile_elements"]): {
        "tags": [VariantTags.MOBILE_ELEMENTS, VariantTags.VCF],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["call_mobile_elements", "call_mobile_elements_index"]): {
        "tags": [VariantTags.MOBILE_ELEMENTS, VariantTags.VCF_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["annotate_mobile_elements", "clinical"]): {
        "tags": [VariantTags.MOBILE_ELEMENTS, AnalysisTags.CLINICAL, VariantTags.VCF],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["annotate_mobile_elements", "clinical_index"]): {
        "tags": [VariantTags.MOBILE_ELEMENTS, AnalysisTags.CLINICAL, VariantTags.VCF_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["annotate_mobile_elements", "research"]): {
        "tags": [VariantTags.MOBILE_ELEMENTS, AnalysisTags.RESEARCH, VariantTags.VCF],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["annotate_mobile_elements", "research_index"]): {
        "tags": [VariantTags.MOBILE_ELEMENTS, AnalysisTags.RESEARCH, VariantTags.VCF_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["call_snv", "call_snv"]): {
        "tags": [VariantTags.VCF_SNV],
        "is_mandatory": True,
        "used_by": [UsageTags.GENOTYPE, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["call_snv", "call_snv_index"]): {
        "tags": [VariantTags.VCF_SNV_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.CG, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["call_snv", "call_snv_mt"]): {
        "tags": [VariantTags.VCF_SNV, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": False,
        "used_by": [UsageTags.CG],
    },
    frozenset(["call_snv", "call_snv_mt_index"]): {
        "tags": [VariantTags.VCF_SNV_INDEX, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": False,
        "used_by": [UsageTags.CG],
    },
    frozenset(["call_sv", "call_sv"]): {
        "tags": [VariantTags.VCF_SV],
        "is_mandatory": False,
        "used_by": [UsageTags.CG, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["call_sv", "call_sv_index"]): {
        "tags": [VariantTags.VCF_SV_INDEX],
        "is_mandatory": False,
        "used_by": [UsageTags.CG, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["call_sv_mt", "call_sv_mt"]): {
        "tags": [VariantTags.VCF_SV, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": False,
        "used_by": [UsageTags.CG],
    },
    frozenset(["call_sv_mt", "call_sv_mt_index"]): {
        "tags": [VariantTags.VCF_SV_INDEX, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": False,
        "used_by": [UsageTags.CG],
    },
    frozenset(["call_sv_mt", "call_sv_mt_del"]): {
        "tags": [AnalysisTags.MITOCHONDRIA, BioinfoToolsTags.MITODEL],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["call_sv_mt", "eklipse_del"]): {
        "tags": [RarediseaseTags.EKLIPSE_DEL, ReportTags.CSV, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["call_sv_mt", "eklipse_png"]): {
        "tags": [RarediseaseTags.EKLIPSE_PNG, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["call_sv_mt", "eklipse_genes"]): {
        "tags": [RarediseaseTags.EKLIPSE_GENES, ReportTags.CSV, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["rank_and_filter", "snv_clinical"]): {
        "tags": [VariantTags.VCF_SNV_CLINICAL],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "snv_clinical_index"]): {
        "tags": [VariantTags.VCF_SNV_CLINICAL_INDEX],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "snv_research"]): {
        "tags": [VariantTags.VCF_SNV_RESEARCH],
        "index_tags": ["vcf-snv-research-index"],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "snv_research_index"]): {
        "tags": [VariantTags.VCF_SNV_RESEARCH_INDEX],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "sv_clinical"]): {
        "tags": [VariantTags.VCF_SV_CLINICAL],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "sv_clinical_index"]): {
        "tags": [VariantTags.VCF_SV_CLINICAL_INDEX],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "sv_research"]): {
        "tags": [VariantTags.VCF_SV_RESEARCH],
        "index_tags": ["vcf-snv-research-index"],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "sv_research_index"]): {
        "tags": [VariantTags.VCF_SV_RESEARCH_INDEX],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "mt_clinical"]): {
        "tags": [VariantTags.VCF_SV_CLINICAL, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "mt_clinical_index"]): {
        "tags": [VariantTags.VCF_SV_CLINICAL_INDEX, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "mt_research"]): {
        "tags": [VariantTags.VCF_SV_RESEARCH, AnalysisTags.MITOCHONDRIA],
        "index_tags": ["vcf-snv-research-index"],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["rank_and_filter", "mt_research_index"]): {
        "tags": [VariantTags.VCF_SV_RESEARCH_INDEX, AnalysisTags.MITOCHONDRIA],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset(["ngsbits_samplegender", "ngsbits_samplegender"]): {
        "tags": [RarediseaseTags.NGSBITS],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["peddy", "peddy"]): {
        "tags": [BioinfoToolsTags.PEDDY, FamilyTags.PED],
        "is_mandatory": True,
        "used_by": [UsageTags.AUDIT, UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["peddy", "ped_check"]): {
        "tags": [BioinfoToolsTags.PEDDY, QCTags.PED_CHECK],
        "is_mandatory": True,
        "used_by": [UsageTags.AUDIT, UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["peddy", "sex_check"]): {
        "tags": [BioinfoToolsTags.PEDDY, QCTags.SEX_CHECK],
        "is_mandatory": True,
        "used_by": [UsageTags.AUDIT, UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["pedigree", "pedigree_fam"]): {
        "tags": [FamilyTags.PEDIGREE],
        "is_mandatory": True,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["annotate_snv", "rhocallviz"]): {
        "tags": [VariantTags.RHOCALL_VIZ],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["annotate_snv_mt", "haplogrep"]): {
        "tags": [RarediseaseTags.HAPLOGREP],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY],
    },
    frozenset(["chromograph_rhoviz", "autozyg"]): {
        "tags": [BioinfoToolsTags.CHROMOGRAPH, AnalysisTags.AUTOZYG],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["chromograph_upd", "regions"]): {
        "tags": [BioinfoToolsTags.CHROMOGRAPH, VariantTags.UPD, AnalysisTags.REGIONS],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset(["chromograph_upd", "sites"]): {
        "tags": [BioinfoToolsTags.CHROMOGRAPH, VariantTags.UPD, AnalysisTags.SITES],
        "is_mandatory": False,
        "used_by": [UsageTags.SCOUT],
    },
    frozenset({"multiqc", "multiqc-html"}): {
        "is_mandatory": True,
        "tags": [ReportTags.MULTIQC_HTML],
        "used_by": [UsageTags.SCOUT, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc", "multiqc-json"}): {
        "is_mandatory": True,
        "tags": [ReportTags.MULTIQC_JSON],
        "used_by": [UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc", "multiqc-general-stats"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.GENERAL_STATS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-alignment"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_ALIGNMENT],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-bamqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_BAMQC],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-insertsize"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_INSERT_SIZE],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-mosdepth-covdist"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.MOSDEPTH_COVDIST],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-mosdepth-cumcov"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.MOSDEPTH_CUMCOV],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-mosdepth-perchrom"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.MOSDEPTH_PERCHROM],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-qualitycycle"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_QUALITYCYCLE],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-qualitydistr"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_QUALITYDISTR],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-peddy"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, BioinfoToolsTags.PEDDY],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-hs"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_HS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-histogram1"}): {
        "is_mandatory": False,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_HISTOGRAM],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-histogram2"}): {
        "is_mandatory": False,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_HISTOGRAM],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-histogram3"}): {
        "is_mandatory": False,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_HISTOGRAM],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-wgs"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_WGS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"nextflow-params"}): {
        "is_mandatory": True,
        "tags": [NextflowTags.NEXTFLOW_PARAMS],
        "used_by": [UsageTags.CG, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"nextflow-config"}): {
        "is_mandatory": True,
        "tags": [NextflowTags.NEXTFLOW_CONFIG],
        "used_by": [UsageTags.CG, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"samplesheet"}): {
        "is_mandatory": True,
        "tags": [NextflowTags.SAMPLESHEET],
        "used_by": [UsageTags.CG, UsageTags.LONG_TERM_STORAGE],
    },
}

RAREDISEASE_TAGS = {**RAREDISEASE_COMMON_TAGS, **NEXTFLOW_TAGS}



"""Store common utilities."""

import json
import logging
from json.decoder import JSONDecodeError
from pathlib import Path

import typer
import yaml
from yaml.parser import ParserError

LOG = logging.getLogger(__name__)


def get_deliverables(deliverables_file: Path) -> dict[str, list[dict[str, str]]]:
    """
    Open file and load into a dictionary. Try to first open YAML, then JSON.

    Raises:
        typer.Abort: If the file cannot be parsed as YAML or JSON.
    """
    with open(deliverables_file, "r") as handle:
        try:
            deliverables = yaml.load(handle, Loader=yaml.SafeLoader)
        except ParserError as error:
            LOG.error(error)
            LOG.warning(f"File {deliverables_file} is not in YAML format")
            try:
                deliverables = json.load(handle)
            except JSONDecodeError as error:
                LOG.error(error)
                LOG.warning(f"File {deliverables_file} is not in JSON format")
                LOG.warning("Can not parse file")
                raise typer.Abort()
        return deliverables



import logging
from enum import StrEnum
from typing import Optional

import coloredlogs
import pkg_resources
import typer

from cg_hermes.cli import convert, export, validate

app = typer.Typer(no_args_is_help=True)

app.add_typer(convert.app, name="convert")
app.add_typer(validate.app, name="validate")
app.add_typer(export.app, name="export")

__version__ = pkg_resources.get_distribution("cg_hermes").version
LOG = logging.getLogger(__name__)


class LogLevel(StrEnum):
    debug = "DEBUG"
    info = "INFO"
    warning = "WARNING"


def version_callback(value: bool):
    if value:
        typer.echo(f"hermes version: {__version__}")
        raise typer.Exit()


@app.callback()
def main(
    loglevel: LogLevel = typer.Option(LogLevel.info, help="Set the log level"),
    version: Optional[bool] = typer.Option(
        None, "--version", callback=version_callback, is_eager=True
    ),
):
    """
    Manage users in the awesome CLI app.
    """
    LOG.info("Running hermes")
    coloredlogs.install(level=loglevel)



from enum import StrEnum


class Workflow(StrEnum):
    BALSAMIC: str = "balsamic"
    BALSAMIC_PON: str = "balsamic-pon"
    BALSAMIC_QC: str = "balsamic-qc"
    BALSAMIC_UMI: str = "balsamic-umi"
    DEMULTIPLEX: str = "demultiplex"
    FASTQ: str = "fastq"
    FLUFFY: str = "fluffy"
    MICROSALT: str = "microsalt"
    MIP_DNA: str = "mip-dna"
    MIP_RNA: str = "mip-rna"
    MUTANT: str = "mutant"
    RAREDISEASE: str = "raredisease"
    RNAFUSION: str = "rnafusion"
    RSYNC: str = "rsync"
    SPRING: str = "spring"
    TAXPROFILER: str = "taxprofiler"
    TOMTE: str = "tomte"

    @classmethod
    def get_nf_workflows(cls) -> set:
        return {cls.RAREDISEASE, cls.RNAFUSION, cls.TAXPROFILER, cls.TOMTE}


class CancerAnalysisType(StrEnum):
    TUMOR_NORMAL_PANEL: str = "tumor_normal_panel"
    TUMOR_NORMAL_WGS: str = "tumor_normal_wgs"
    TUMOR_PANEL: str = "tumor_panel"
    TUMOR_WGS: str = "tumor_wgs"



MICROSALT_COMMON_TAGS = {
    frozenset({"sampleinfo", "analysis"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": ["config"],
        "used_by": ["storage", "audit"],
    },
    frozenset({"microsalt-qc", "result_aggregation"}): {
        "is_mandatory": True,
        "tags": ["qc-report", "visualization"],
        "used_by": ["deliver"],
    },
    frozenset({"microsalt-type", "result_aggregation"}): {
        "is_mandatory": False,
        "tags": ["typing-report", "visualization"],
        "used_by": ["deliver"],
    },
    frozenset({"microsalt-json", "result_aggregation"}): {
        "is_mandatory": False,
        "tags": ["typing-report", "qc-metrics"],
        "used_by": ["storage"],
    },
    frozenset({"runtime-settings", "analysis"}): {
        "is_mandatory": True,
        "tags": ["microsalt-config"],
        "used_by": ["storage", "audit"],
    },
    frozenset({"assembly"}): {"is_mandatory": False, "tags": ["assembly"], "used_by": ["storage"]},
    frozenset({"concatination", "trimmed-forward-reads"}): {
        "is_mandatory": False,
        "tags": ["fastq", "forward-strand"],
        "used_by": ["storage"],
    },
    frozenset({"trimmed-reverse-reads", "concatination"}): {
        "is_mandatory": False,
        "tags": ["fastq", "reverse-strand"],
        "used_by": ["storage"],
    },
    frozenset({"trimmed-unpaired-reads", "concatination"}): {
        "is_mandatory": False,
        "tags": ["fastq", "unpaired-reads"],
        "used_by": ["storage"],
    },
    frozenset({"analysis", "logfile"}): {
        "is_mandatory": False,
        "tags": ["microsalt-log"],
        "used_by": ["audit"],
    },
    frozenset({"quast-results", "assembly"}): {
        "is_mandatory": False,
        "tags": ["qc-metrics", "assembly"],
        "used_by": ["audit"],
    },
    frozenset({"alignment", "reference-alignment-sorted"}): {
        "is_mandatory": False,
        "tags": ["bam"],
        "used_by": ["storage"],
    },
    frozenset({"picard-insertsize", "insertsize_calc"}): {
        "is_mandatory": False,
        "tags": ["qc-metrics", "picard"],
        "used_by": ["audit"],
    },
}



import logging
from pathlib import Path

import typer
from pydantic import ValidationError

from cg_hermes.cli.common import get_deliverables
from cg_hermes.config.balsamic import BALSAMIC_TAGS
from cg_hermes.config.balsamic_qc import BALSAMIC_QC_TAGS
from cg_hermes.config.balsamic_umi import BALSAMIC_UMI_TAGS
from cg_hermes.config.fluffy import FLUFFY_COMMON_TAGS
from cg_hermes.config.mip_dna import MIP_DNA_TAGS
from cg_hermes.config.mip_rna import MIP_RNA_TAGS
from cg_hermes.config.mutant import MUTANT_COMMON_TAGS
from cg_hermes.config.raredisease import RAREDISEASE_TAGS
from cg_hermes.config.rnafusion import RNAFUSION_TAGS
from cg_hermes.config.taxprofiler import TAXPROFILER_TAGS
from cg_hermes.config.tomte import TOMTE_TAGS
from cg_hermes.constants.workflow import CancerAnalysisType, Workflow
from cg_hermes.deliverables import Deliverables
from cg_hermes.exceptions import MissingFileError
from cg_hermes.validate import get_deliverables_obj, validate_tag_map

LOG = logging.getLogger(__name__)

app = typer.Typer()


@app.command("deliverables")
def validate_deliverables(
    deliverables_file: Path,
    workflow: Workflow = typer.Option(Workflow.FLUFFY, help="Specify workflow"),
    analysis_type: CancerAnalysisType = typer.Option(None, help="Specify the analysis type"),
    force: bool = typer.Option(False, "--force", "-f", help="Bypass deliverables file validation"),
) -> None:
    """
    Validate a deliverables file.

    Raises:
        typer.Abort: If there is an error in parsing or validating the deliverables file.
    """
    LOG.info(f"Validating file: {deliverables_file} from workflow: {workflow}")
    raw_deliverables: dict[str, list[dict[str, str]]] = get_deliverables(deliverables_file)
    try:
        deliverables: Deliverables = get_deliverables_obj(
            deliverables=raw_deliverables, workflow=workflow, analysis_type=analysis_type
        )
        deliverables.validate_mandatory_files(force)
    except SyntaxError as error:
        LOG.error(error)
        raise typer.Abort()
    except (ValidationError, MissingFileError) as error:
        LOG.error(error)
        LOG.error(f"File: {deliverables_file} does not follow the specification")
        raise typer.Abort()
    LOG.info("Deliverables file has the correct format")


@app.command("tags")
def validate_tags_cmd(workflow: Workflow) -> None:
    """Validate the tag maps for one of the definitions."""
    LOG.info(f"Validating {workflow} common tags")
    exit_code = 0

    if workflow == Workflow.MIP_DNA:
        tag_map = MIP_DNA_TAGS
    elif workflow == Workflow.MIP_RNA:
        tag_map = MIP_RNA_TAGS
    elif workflow == Workflow.BALSAMIC:
        tag_map = BALSAMIC_TAGS
    elif workflow == Workflow.BALSAMIC_UMI:
        tag_map = BALSAMIC_UMI_TAGS
    elif workflow == Workflow.BALSAMIC_QC:
        tag_map = BALSAMIC_QC_TAGS
    elif workflow == Workflow.FLUFFY:
        tag_map = FLUFFY_COMMON_TAGS
    elif workflow == Workflow.MUTANT:
        tag_map = MUTANT_COMMON_TAGS
    elif workflow == Workflow.RAREDISEASE:
        tag_map = RAREDISEASE_TAGS
    elif workflow == Workflow.RNAFUSION:
        tag_map = RNAFUSION_TAGS
    elif workflow == Workflow.TAXPROFILER:
        tag_map = TAXPROFILER_TAGS
    elif workflow == Workflow.TOMTE:
        tag_map = TOMTE_TAGS
    else:
        LOG.info(f"Could not find workflow tags for {workflow}")
        raise typer.Exit(code=exit_code)

    try:
        validate_tag_map(tag_map=tag_map)
        LOG.info("Tag map looks fine")
    except ValidationError as error:
        LOG.warning(error)
        exit_code = 1
    raise typer.Exit(code=exit_code)



"""Tags that are defined in Balsamic UMI deliverables mapped to tags used in CG

The tag sets that exists in all files are set to mandatory. Tag sets that exists in < 4 deliverables are not
mandatory by default. However the tags that are available to a particular analysis is mandatory for that analysis.
"""

from cg_hermes.config.balsamic import BALSAMIC_TAGS, RAW_TAGS

UMI_ALIGNMENT_TAGS = {
    # Alignment files
    frozenset(RAW_TAGS["tumor_umi_consensusfiltered.merged.cram"]): {  # UMI cram (tumor)
        "tags": ["umi-cram", "tumor"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["tumor_umi_consensusfiltered.merged.cram.crai"]): {
        "tags": ["umi-cram-index", "tumor"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["normal_umi_consensusfiltered.merged.cram"]): {  # UMI cram (normal)
        "tags": ["umi-cram", "normal"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["normal_umi_consensusfiltered.merged.cram.crai"]): {
        "tags": ["umi-cram-index", "normal"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
}

UMI_CALLERS_TAGS = {
    # SNVs/INDELs
    frozenset(RAW_TAGS["tnscope_umi.vcf.gz"]): {
        "tags": ["tnscope-umi", "vcf-umi-snv"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tnscope_umi.vcf.gz.tbi"]): {
        "tags": ["tnscope-umi", "vcf-umi-snv-index"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tnscope_umi.research.filtered.pass.vcf.gz"]): {
        "tags": ["tnscope-umi", "vcf-umi-snv-research"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tnscope_umi.research.filtered.pass.vcf.gz.tbi"]): {
        "tags": ["tnscope-umi", "vcf-umi-snv-research-index"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tnscope_umi.clinical.filtered.pass.vcf.gz"]): {
        "tags": ["tnscope-umi", "vcf-umi-snv-clinical"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["tnscope_umi.clinical.filtered.pass.vcf.gz.tbi"]): {
        "tags": ["tnscope-umi", "vcf-umi-snv-clinical-index"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    # TMB
    frozenset(RAW_TAGS["tnscope_umi.balsamic_stat"]): {
        "tags": ["research", "tnscope-umi", "tmb"],
        "is_mandatory": True,
        "used_by": ["storage"],
    },
}

BALSAMIC_UMI_TAGS = {**BALSAMIC_TAGS, **UMI_ALIGNMENT_TAGS, **UMI_CALLERS_TAGS}

UMI_TUMOR_ONLY_PANEL_TAGS = {
    # SNVs/INDELs (PANEL)
    frozenset(RAW_TAGS["vardict.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.research.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.research.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.clinical.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.clinical.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    # CNVs (PANEL)
    frozenset(RAW_TAGS["tumor.merged.cns"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tumor.merged.cnr"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["gene_metrics"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["cnvkit.vcf2cytosure.cgh"]): {"is_mandatory": True},
    # TMB
    frozenset(RAW_TAGS["vardict.balsamic_stat"]): {"is_mandatory": True},
}

UMI_TUMOR_NORMAL_PANEL_TAGS = {
    # Alignment files (PANEL & WGS)
    frozenset(RAW_TAGS["normal.merged.cram"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["normal.merged.cram.crai"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["normal_umi_consensusfiltered.merged.cram"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["normal_umi_consensusfiltered.merged.cram.crai"]): {"is_mandatory": True},
    # Germline SNVs (PANEL & WGS)
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz.tbi"]): {"is_mandatory": True},
    # SNVs/INDELs (PANEL)
    frozenset(RAW_TAGS["vardict.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.research.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.research.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.clinical.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.clinical.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    # CNVs (PANEL)
    frozenset(RAW_TAGS["tumor.merged.cns"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tumor.merged.cnr"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["gene_metrics"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["cnvkit.vcf2cytosure.cgh"]): {"is_mandatory": True},
    # TMB
    frozenset(RAW_TAGS["vardict.balsamic_stat"]): {"is_mandatory": True},
}



"""Tags that are defined in nextflow deliverables mapped to tags used in CG

The tag sets that exists in all files are set to mandatory. Tag sets that exists in < 4 deliverables are not
mandatory by default. However, the tags that are available to a particular analysis is mandatory for that analysis.
"""

from cg_hermes.constants.tags import UsageTags, NextflowTags

from cg_hermes.constants.tags import (
    NextflowTags,
    QCTags,
    ReportTags,
    UsageTags,
)


NEXTFLOW_TAGS = {
    frozenset({"software-versions"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [NextflowTags.SOFTWARE_VERSIONS],
        "used_by": [UsageTags.CG, UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({QCTags.QC_METRICS}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.DELIVERABLE],
        "used_by": [UsageTags.CG, UsageTags.LONG_TERM_STORAGE],
    },
}
